==> ICA-1.5.tar.gz: ICA-1.5/src/MSI/Spec2Gen/callbacks.c <==
/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      $RCSfile: callbacks.c,v $
   Purpose:     The callbacks for various parts of the interface.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    $Revision: 1.7 $
   Date:        $Date: 92/05/20 11:47:28 $

   HISTORY
   $Log:	callbacks.c,v $
 * Revision 1.7  92/05/20  11:47:28  ware
 * 
 * For some reason, certain window manager settings prevent the event
 * handler from ever getting called.  I don't quite understand why.  What
 * I do know is that making it a translation fixes the problem.  That is
 * about all that happens.  I change SymbolPlace from being an event
 * handler to being an action proc.
 * 
 * Revision 1.6  92/03/16  13:01:46  ware
 * Conditionalize the title
 * 
 * Revision 1.5  1992/02/21  06:06:58  cso
 * Aesthetics
 *
 * Revision 1.4  1992/02/11  22:48:43  cso
 * Fixed the call to GetFilename/OpenFile and fixed the bogus load spec msg.
 *
 * Revision 1.3  1992/01/14  20:53:27  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:24:31  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:53:52  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "fileio.h"
#include "generate.h"
#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "permshell.h"
#include "status.h"
#include "utils.h"

/*  GLOBALS  */




/*
   FUNCTION: 		DeletePermSymbol

   Synopsis: 
       Delete the perm symbol and put it back in the production.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
DeletePermSymbol(w, symbol, call_data)

Widget
   w;
SymbolList
   *symbol;
XtPointer
   call_data;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   PermutationList
      *plptr;
   RhsList
      *rlptr;
   SymbolList
      *slptr;
   

   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   if (symbol == NULL)
   {
      return;
   }
#ifdef DEBUG
   fprintf(stderr, "We need to kill [%s]...\n", symbol->symbolstr);
#endif
      
   
   plptr = rlptr->activeperm;
   for (slptr = rlptr->symbol; slptr != NULL; slptr = slptr->next)
   {
      if (slptr->position == symbol->position)
      {
	 XtSetSensitive(slptr->symbutton, True);
	 XawToggleSetCurrent(slptr->symbutton, slptr);
	 XtDestroyWidget(w);
	 symbol->symbutton = NULL;
	 break;
      }
   }
   
   /* fix the pointers */
   if (symbol->prev != NULL)
   {
      symbol->prev->next = symbol->next;
   }
   else
   {
      plptr->permsymbol = symbol->next;
   }
   if (symbol->next != NULL)
   {
      symbol->next->prev = symbol->prev;
      
      /* and fix the fromHoriz of the widget to our right */
      XtUnmanageChild(symbol->next->symbutton);
      cnt = 0;
      if (symbol->prev != NULL)
      {
	 XtSetArg(args[cnt], XtNfromHoriz, symbol->prev->symbutton); 
      }
      else
      {
	 XtSetArg(args[cnt], XtNfromHoriz, NULL); 
      }
      cnt++;
      XtSetValues(symbol->next->symbutton, args, cnt);
      XtManageChild(symbol->next->symbutton);
   }
   
   /* and free up the node */
   XtFree(symbol->symbolstr);
   XtFree(symbol->occurstr);
   XtFree(symbol);

   UpdateInterface(Gindex, Gindex);
   Dirtyflag = True;

   return;
}



/*
   FUNCTION: 		SymbolPlace

   Synopsis: 
       Place a symbol in the permutation order.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
SymbolPlace(w, event, params, num)

Widget
   w;
XEvent
   *event;
String
   *params;
int
   num;

{
   PermutationList
      *plptr;
   RhsList 
      *rlptr;
   SymbolList
      *leftslptr,
      *slptr,   /* symbol list ptr for a rhs */
      *splptr;  /* symbol list ptr for a permutation */

   /* check if button 1 was released inside window */
   if (event->xbutton.button != Button1)
   {
      return;
   }
   
   /* find what alt is active */
   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   slptr = (SymbolList *) XawToggleGetCurrent(rlptr->symbol->symbutton);
   
   if (slptr == NULL)
   {
      if (rlptr->symbol->next == NULL)
      {
	 slptr = rlptr->symbol;
      }
      else
      {
	 return;
      }
   }
   
#ifdef DEBUG
   fprintf(stderr, "Found ptr [%s]\n", slptr->symbolstr);
#endif
   if (rlptr->perminstance == NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "SymbolPlace: NULL perminstancen");
#endif
      return;
   }
   
   /* find which permutation is active....*/
   plptr = rlptr->activeperm;
   if (plptr == NULL)
   {
      return;
   }
   
   /* Create a new symbol list for it */
   splptr              = XtNew(SymbolList);
   splptr->symbolstr   = XtNewString(slptr->symbolstr);
   splptr->occurstr    = XtNewString(slptr->occurstr);
   splptr->position    = slptr->position;
   
   /*
    * If I don't have a symbol list to plptrace the new guy
    * then make him my first and only symbol so far
    */
   if (plptr->permsymbol == NULL)
   {
      plptr->permsymbol   = splptr;
      splptr->next        = NULL;
      splptr->prev        = NULL;
   }
   else
   {
      /*
       * Find the left sibling for the new guy
       * Assign it to splptr->previous
       * if the left sibling is NULL, i.e., head of list
       * splptr becomes the head of the list
       * else just fix it as if it is in the middle
       */
      leftslptr    = FindLeftSibling(plptr, event->xbutton.x);
      splptr->prev = leftslptr;
      if (leftslptr == NULL)
      {
	 splptr->next      = plptr->permsymbol;
	 plptr->permsymbol = splptr;
      }
      else
      {
	 /*
	  * Check if splptr is at the tail
	  * If not then fix the next guy to point back to me
	  */
	 splptr->next    = leftslptr->next;
	 leftslptr->next = splptr;
      }
      if (splptr->next != NULL)
      {
	 splptr->next->prev = splptr;
      }
   }
   MakePermButton(splptr, splptr->prev, splptr->next);
   XtSetSensitive(slptr->symbutton, False);
   UpdatePermlist();

   Dirtyflag = True;

   return;
}



/*
   FUNCTION: 		AltMove

   Synopsis: 
       Move in the production of a symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
AltMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   RhsList
      *rlptr;
   Widget
      prodlist;
   XawListReturnStruct
      *current;
   
   /* find what alternative production of the rhs is active */
   prodlist = XtNameToWidget(Toplevel, "*prodlist");
   if (prodlist == NULL)
   {
      return;
   }
   
   current = XawListShowCurrent(prodlist);
   XawListUnhighlight(prodlist, current->list_index);
   
   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "Warning: AltMove called with bad value!!\n");
#endif
      break;
      
   case Previous:
      /* try to move to the previous alternative */
      if (rlptr->prev != NULL)
      {
	 rlptr = rlptr->prev;
	 current->list_index--;
      }
      break;
      
   case Next:
      /* or, try to move to the next alternative */
      if (rlptr->next != NULL)
      {
	 rlptr = rlptr->next;
	 current->list_index++;
      }
      break;
   }

   Grammar[Gindex].activerhs = rlptr;
   
   XawListHighlight(prodlist, current->list_index);
   AdjustVerticalScrollbar(current->list_index);
      
   UpdateInterface(Gindex, Gindex);
   return;
}



/*
   FUNCTION: 		PermMove

   Synopsis: 
       Move to another permutation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PermMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   PermutationList
      *plptr;
   RhsList
      *rlptr;

   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   plptr = rlptr->activeperm;
   if (plptr == NULL)
   {
      return;
   }
   
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "Warning: AltMove called with bad value!!\n");
#endif
      break;
      
   case Previous:
      /* try to move to the previous alternative */
      if (plptr->prev != NULL)
      {
	 plptr = plptr->prev;
	 rlptr->activeperm = plptr;
      }
      break;
      
   case Next:
      /* or, try to move to the next alternative */
      if (plptr->next != NULL)
      {
	 plptr = plptr->next;
	 rlptr->activeperm = plptr;
      }
      break;
   }
   UpdateInterface(Gindex, Gindex);
   return;
}



/*
   FUNCTION: 		ProdMove

   Synopsis: 
       Move to another production.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ProdMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   int
      oldindex;
   
   oldindex = Gindex;
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "ProdMove passed an illegal direction\n");
#endif
      break;
      
   case Next:
      /* move us to the next production */
      if (Gindex < Gmax - 1)
      {
	 Gindex++;
#ifdef DEBUG
	 fprintf(stderr, "Gindex now = %d\n\n", Gindex);
#endif
      }
      break;
      
   case Previous:
      /* move us to the previous production */
      if (Gindex > 0)
      {
	 Gindex--;
#ifdef DEBUG
	 fprintf(stderr, "Gindex now = %d\n\n", Gindex);
#endif
      }
      break;
   }
   if (oldindex != Gindex)
   {
      UpdateInterface(oldindex, Gindex);
   }
   
   return;
}



/*
   FUNCTION: 		MakeApply

   Synopsis: 
       Make a translator with the right settings.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakeApply(w, panel, call_data)

Widget
   w,
   panel;
XtPointer
   call_data;

{
   FILE
      *lexptr,
      *yaccptr;
   String
      target;
   SymbolTable
      *tokentab;
   UpInterface
      granularity,
      pitype,
      uitype;
   
   
   if (Gmax == 0)
   {
      XicaStatus(Statw, "Please load a specification file first.\n");
      return;
   }
   
   XtSetSensitive(w, False);

   target = GetFilename("Making a specific to general translator [.sgx]",
			"Make:",
			"Make",
			"*.sgx",
			"sgx",
			False);

   if (*target == '\0')
   {
      XtSetSensitive(w, True);
      return;
   }
   
   uitype = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*autotog"));
   granularity = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*elementtog"));
   pitype = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*removetog"));

   lexptr  = OpenGenerationFile(target, False);
   yaccptr = OpenGenerationFile(target, True);

   tokentab = GenerateYacc(yaccptr);
   fclose(yaccptr);
   GenerateLex(lexptr, tokentab, pitype);
   fclose(lexptr);
   
   MakeTranslator(target, uitype, granularity);

   XtFree(target);

   XtSetSensitive(w, True);
   return;
}



/*
   FUNCTION: 		MakeDefaults

   Synopsis: 
       Reset make panel to default settings.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakeDefaults(w, panel, call_data)

Widget
   w,
   panel;
XtPointer
   call_data;

{
   Widget
      autotog,
      elementtog,
      removetog;
   
   autotog    = XtNameToWidget(panel, "*autotog");
   elementtog = XtNameToWidget(panel, "*elementtog");
   removetog  = XtNameToWidget(panel, "*removetog");
   if ((autotog != NULL) && (removetog != NULL) && (elementtog != NULL))
   {
      XawToggleSetCurrent(autotog,    Without);
      XawToggleSetCurrent(removetog,  Without);
      XawToggleSetCurrent(elementtog, Without);
   }
   return;
}

==> ICA-1.6.tar.gz: ICA-1.6/src/MSI/Spec2Gen/callbacks.c <==
/*
   Owner:       Copyright 1992 The Ohio State University,
                Integrated Chameleon Architecture
   Module:      callbacks.c,v
   Purpose:     The callbacks for various parts of the interface.

   Programmer:  Conleth O'Connell
   Reviewer:    
   Revision:    1.8
   Date:        1993/04/19 17:08:17

   HISTORY
   callbacks.c,v
 * Revision 1.8  1993/04/19  17:08:17  ware
 * Merged John Boyds changes.
 *
 * Revision 1.7  92/05/20  11:47:28  ware
 * 
 * For some reason, certain window manager settings prevent the event
 * handler from ever getting called.  I don't quite understand why.  What
 * I do know is that making it a translation fixes the problem.  That is
 * about all that happens.  I change SymbolPlace from being an event
 * handler to being an action proc.
 * 
 * Revision 1.6  92/03/16  13:01:46  ware
 * Conditionalize the title
 * 
 * Revision 1.5  1992/02/21  06:06:58  cso
 * Aesthetics
 *
 * Revision 1.4  1992/02/11  22:48:43  cso
 * Fixed the call to GetFilename/OpenFile and fixed the bogus load spec msg.
 *
 * Revision 1.3  1992/01/14  20:53:27  cso
 * Connected up the code generation with the interface.
 *
 * Revision 1.2  1992/01/10  02:24:31  cso
 * Visual code inspection changes, and the first steps of integrating the
 * functionality of the up translator generator.
 *
 * Revision 1.1  1992/01/08  19:53:52  cso
 * Initial revision
 *

   Functions Index:
   AVAIL   NAME DEPENDS ON INTERNALLY      EXTERNALLY
   -----   ----            ----------      ----------

*/

/* #INCLUDES */

#include "widgets.h"
#include "types.h"
#include "defs.h"

/* #DEFINES  */

/*  EXTERNS  */

#include "display.h"
#include "fileio.h"
#include "generate.h"
#include "initialize.h"
#include "interface.h"
#include "main.h"
#include "permshell.h"
#include "status.h"
#include "utils.h"

/*  GLOBALS  */




/*
   FUNCTION: 		DeletePermSymbol

   Synopsis: 
       Delete the perm symbol and put it back in the production.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
DeletePermSymbol(w, symbol, call_data)

Widget
   w;
SymbolList
   *symbol;
XtPointer
   call_data;

{
   Arg
      args[MAXARGS];
   int
      cnt;
   PermutationList
      *plptr;
   RhsList
      *rlptr;
   SymbolList
      *slptr;
   

   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   if (symbol == NULL)
   {
      return;
   }
#ifdef DEBUG
   fprintf(stderr, "We need to kill [%s]...\n", symbol->symbolstr);
#endif
      
   
   plptr = rlptr->activeperm;
   for (slptr = rlptr->symbol; slptr != NULL; slptr = slptr->next)
   {
      if (slptr->position == symbol->position)
      {
	 XtSetSensitive(slptr->symbutton, True);
	 XawToggleSetCurrent(slptr->symbutton, slptr);
	 XtDestroyWidget(w);
	 symbol->symbutton = NULL;
	 break;
      }
   }
   
   /* fix the pointers */
   if (symbol->prev != NULL)
   {
      symbol->prev->next = symbol->next;
   }
   else
   {
      plptr->permsymbol = symbol->next;
   }
   if (symbol->next != NULL)
   {
      symbol->next->prev = symbol->prev;
      
      /* and fix the fromHoriz of the widget to our right */
      XtUnmanageChild(symbol->next->symbutton);
      cnt = 0;
      if (symbol->prev != NULL)
      {
	 XtSetArg(args[cnt], XtNfromHoriz, symbol->prev->symbutton); 
      }
      else
      {
	 XtSetArg(args[cnt], XtNfromHoriz, NULL); 
      }
      cnt++;
      XtSetValues(symbol->next->symbutton, args, cnt);
      XtManageChild(symbol->next->symbutton);
   }
   
   /* and free up the node */
   XtFree(symbol->symbolstr);
   XtFree(symbol->occurstr);
   XtFree((char *) symbol);

   UpdateInterface(Gindex, Gindex);
   Dirtyflag = True;

   return;
}



/*
   FUNCTION: 		SymbolPlace

   Synopsis: 
       Place a symbol in the permutation order.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/



void
SymbolPlace(w, event, params, num)

Widget
   w;
XEvent
   *event;
String
   *params;
int
   num;

{
   PermutationList
      *plptr;
   RhsList 
      *rlptr;
   SymbolList
      *leftslptr,
      *slptr,   /* symbol list ptr for a rhs */
      *splptr;  /* symbol list ptr for a permutation */

   /* check if button 1 was released inside window */
   if (event->xbutton.button != Button1)
   {
      return;
   }
   
   /* find what alt is active */
   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   slptr = (SymbolList *) XawToggleGetCurrent(rlptr->symbol->symbutton);
   
   if (slptr == NULL)
   {
      if (rlptr->symbol->next == NULL)
      {
	 slptr = rlptr->symbol;
      }
      else
      {
	 return;
      }
   }
   
#ifdef DEBUG
   fprintf(stderr, "Found ptr [%s]\n", slptr->symbolstr);
#endif
   if (rlptr->perminstance == NULL)
   {
#ifdef DEBUG
      fprintf(stderr, "SymbolPlace: NULL perminstancen");
#endif
      return;
   }
   
   /* find which permutation is active....*/
   plptr = rlptr->activeperm;
   if (plptr == NULL)
   {
      return;
   }
   
   /* Create a new symbol list for it */
   splptr              = XtNew(SymbolList);
   splptr->symbolstr   = XtNewString(slptr->symbolstr);
   splptr->occurstr    = XtNewString(slptr->occurstr);
   splptr->position    = slptr->position;
   
   /*
    * If I don't have a symbol list to plptrace the new guy
    * then make him my first and only symbol so far
    */
   if (plptr->permsymbol == NULL)
   {
      plptr->permsymbol   = splptr;
      splptr->next        = NULL;
      splptr->prev        = NULL;
   }
   else
   {
      /*
       * Find the left sibling for the new guy
       * Assign it to splptr->previous
       * if the left sibling is NULL, i.e., head of list
       * splptr becomes the head of the list
       * else just fix it as if it is in the middle
       */
      leftslptr    = FindLeftSibling(plptr, event->xbutton.x);
      splptr->prev = leftslptr;
      if (leftslptr == NULL)
      {
	 splptr->next      = plptr->permsymbol;
	 plptr->permsymbol = splptr;
      }
      else
      {
	 /*
	  * Check if splptr is at the tail
	  * If not then fix the next guy to point back to me
	  */
	 splptr->next    = leftslptr->next;
	 leftslptr->next = splptr;
      }
      if (splptr->next != NULL)
      {
	 splptr->next->prev = splptr;
      }
   }
   MakePermButton(splptr, splptr->prev, splptr->next);
   XtSetSensitive(slptr->symbutton, False);
   UpdatePermlist();

   Dirtyflag = True;

   return;
}



/*
   FUNCTION: 		AltMove

   Synopsis: 
       Move in the production of a symbol.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/


void
AltMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   RhsList
      *rlptr;
   Widget
      prodlist;
   XawListReturnStruct
      *current;
   
   /* find what alternative production of the rhs is active */
   prodlist = XtNameToWidget(Toplevel, "*prodlist");
   if (prodlist == NULL)
   {
      return;
   }
   
   current = XawListShowCurrent(prodlist);
   XawListUnhighlight(prodlist);
   
   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "Warning: AltMove called with bad value!!\n");
#endif
      break;
      
   case Previous:
      /* try to move to the previous alternative */
      if (rlptr->prev != NULL)
      {
	 rlptr = rlptr->prev;
	 current->list_index--;
      }
      break;
      
   case Next:
      /* or, try to move to the next alternative */
      if (rlptr->next != NULL)
      {
	 rlptr = rlptr->next;
	 current->list_index++;
      }
      break;
   }

   Grammar[Gindex].activerhs = rlptr;
   
   XawListHighlight(prodlist, current->list_index);
   AdjustVerticalScrollbar(current->list_index);
      
   UpdateInterface(Gindex, Gindex);
   return;
}



/*
   FUNCTION: 		PermMove

   Synopsis: 
       Move to another permutation.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
PermMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   PermutationList
      *plptr;
   RhsList
      *rlptr;

   rlptr = Grammar[Gindex].activerhs;
   if (rlptr == NULL)
   {
      return;
   }
   
   plptr = rlptr->activeperm;
   if (plptr == NULL)
   {
      return;
   }
   
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "Warning: AltMove called with bad value!!\n");
#endif
      break;
      
   case Previous:
      /* try to move to the previous alternative */
      if (plptr->prev != NULL)
      {
	 plptr = plptr->prev;
	 rlptr->activeperm = plptr;
      }
      break;
      
   case Next:
      /* or, try to move to the next alternative */
      if (plptr->next != NULL)
      {
	 plptr = plptr->next;
	 rlptr->activeperm = plptr;
      }
      break;
   }
   UpdateInterface(Gindex, Gindex);
   return;
}



/*
   FUNCTION: 		ProdMove

   Synopsis: 
       Move to another production.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
ProdMove(w, dir, call_data)

Widget
   w;           /* Unused */
Direction
   dir;		/* direction of movement (previous or next) */
caddr_t
   call_data;   /* Unused */

{
   int
      oldindex;
   
   oldindex = Gindex;
   switch (dir)
   {
   default:
#ifdef DEBUG
      fprintf(stderr, "ProdMove passed an illegal direction\n");
#endif
      break;
      
   case Next:
      /* move us to the next production */
      if (Gindex < Gmax - 1)
      {
	 Gindex++;
#ifdef DEBUG
	 fprintf(stderr, "Gindex now = %d\n\n", Gindex);
#endif
      }
      break;
      
   case Previous:
      /* move us to the previous production */
      if (Gindex > 0)
      {
	 Gindex--;
#ifdef DEBUG
	 fprintf(stderr, "Gindex now = %d\n\n", Gindex);
#endif
      }
      break;
   }
   if (oldindex != Gindex)
   {
      UpdateInterface(oldindex, Gindex);
   }
   
   return;
}



/*
   FUNCTION: 		MakeApply

   Synopsis: 
       Make a translator with the right settings.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakeApply(w, panel, call_data)

Widget
   w,
   panel;
XtPointer
   call_data;

{
   FILE
      *lexptr,
      *yaccptr;
   String
      target;
   SymbolTable
      *tokentab;
   UpInterface
      granularity,
      pitype,
      uitype;
   
   
   if (Gmax == 0)
   {
      XicaStatus(Statw, "Please load a specification file first.\n");
      return;
   }
   
   XtSetSensitive(w, False);

   target = GetFilename("Making a specific to general translator [.sgx]",
			"Make:",
			"Make",
			"*.sgx",
			"sgx",
			False);

   if (*target == '\0')
   {
      XtSetSensitive(w, True);
      return;
   }
   
   uitype = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*autotog"));
   granularity = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*elementtog"));
   pitype = (UpInterface)
      XawToggleGetCurrent(XtNameToWidget(panel, "*removetog"));

   lexptr  = OpenGenerationFile(target, False);
   yaccptr = OpenGenerationFile(target, True);

   tokentab = GenerateYacc(yaccptr);
   fclose(yaccptr);
   GenerateLex(lexptr, tokentab, pitype);
   fclose(lexptr);
   
   MakeTranslator(target, uitype, granularity);

   XtFree(target);

   XtSetSensitive(w, True);
   return;
}



/*
   FUNCTION: 		MakeDefaults

   Synopsis: 
       Reset make panel to default settings.

   Initial purpose:
       Same as above.

   Present purpose:
       Same as above.

   Creator:	Conleth O'Connell
   Reviewer:	

   Modifications:
       DATE     WHO      REASON

   Globals used:
       none

   Globals Modified:
       none

   Return value:	void

   Range:		none
*/

void
MakeDefaults(w, panel, call_data)

Widget
   w,
   panel;
XtPointer
   call_data;

{
   Widget
      autotog,
      elementtog,
      removetog;
   
   autotog    = XtNameToWidget(panel, "*autotog");
   elementtog = XtNameToWidget(panel, "*elementtog");
   removetog  = XtNameToWidget(panel, "*removetog");
   if ((autotog != NULL) && (removetog != NULL) && (elementtog != NULL))
   {
      XawToggleSetCurrent(autotog,    (XtPointer) Without);
      XawToggleSetCurrent(removetog,  (XtPointer) Without);
      XawToggleSetCurrent(elementtog, (XtPointer) Without);
   }
   return;
}
